// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - PERK FUNCTIONALITY
// Contains functionality related to perks
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define PERKS_BASEXP						100
#define PERKS_XP_PER_LEVEL					8.30	// How much to mult XP by per level

// ------------------------/
// Can perks be switched?
// This only applies to the mid-game menu
//
// SERVER
// ------------------------/

bool PerkSwitchAllowed()
{
	if (GameState.State != STATE_TRADER && GameState.State != STATE_SANDBOX)
		return false;
		
	// Just in case
	if (!PlayerState[PlayerNumber()].Alive)
		return false;
		
	return true;
}

// ------------------------/
// Give some experience!
// Requires player as ACTIVATOR!
// ------------------------/

void GiveXP(int xpAmount)
{
	// Not a player! Cannot give XP
	if (!(ClassifyActor(0) & ACTOR_PLAYER))
		return;
		
	// Would giving us the experience level us up?
	int xpMax = GetAmmoCapacity("PerkXP");
	int xpCur = CheckInventory("PerkXP");
	
	// Perform a loop
	// (This could level us up multiple times)
	
	if (xpCur + xpAmount >= xpMax)
	{
		int oldLevel = CheckInventory("PerkLevel");
		
		while (xpAmount > 0)
		{
			xpMax = GetAmmoCapacity("PerkXP");
			xpCur = CheckInventory("PerkXP");
			
			// Would level up?
			if (xpCur + xpAmount >= xpMax)
			{
				// Subtract what's left
				xpAmount -= (xpMax - xpCur);
				GiveInventory("PerkLevel", 1);
				
				// Set our new XP amount
				SetAmmoCapacity("PerkXP", GetXPGoal( CheckInventory("PerkLevel") ));
				
				InvEnsure("PerkXP", 0);
			}
			
			// Wouldn't level us up, just give what's left
			else
			{
				GiveInventory("PerkXP", xpAmount);
				xpAmount = 0;
			}
		}
		
		int newLevel = CheckInventory("PerkLevel");
		
		int OID = ActivatorTID();
		int PN = PlayerNumber();
		
		// This will get sync'd to clients
		PlayerState[PN].PerkLevel = newLevel;
		
		// SetActivator(-1);
		Rep_UpdatePlayerState(PN, -1);
		// SetActivator(OID);
		
		LeveledUp(newLevel - oldLevel);
	}
	
	// Wouldn't level us up, just give it
	else
		GiveInventory("PerkXP", xpAmount);
}

// ------------------------/
// Get a perk XP goal for a specific level
// ------------------------/

int GetXPGoal(int level)
{
	int baseXP = PERKS_BASEXP * 1.0;
	int mult = FixedMul(level * 1.0, PERKS_XP_PER_LEVEL);
	
	return ShaveInt( FixedMul(baseXP, mult) );
}

// ------------------------/
// We leveled up!
// ------------------------/

void LeveledUp(int levels)
{
	SetFont(FNT_PLAIN);
	Print(s:"YOU LEVELED UP");
}

// ------------------------/
// Perk we're currently using
// ------------------------/

int OurPerk()
{
	int PN = IsLocal() ? ConsolePlayerNumber() : PlayerNumber();
	return PlayerState[ PN ].PerkIndex;
}