// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - PLAYER
// Contains player-related functions
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// ------------------------/
// Setup some required player things for an ACTUAL PLAYER
// This is done when they "spawn" into the game
// ------------------------/

void SetupPhysicalPlayer()
{
	// Set weapon list to junk so it's forced to change
	int PN = PlayerNumber();
	
	PlayerState[PN].Ready = true;
	PlayerState[PN].Alive = true;
	
	// Assign a TID
	Thing_ChangeTID(0,BASE_PLAYER_TID+PN);
	
	curWeaponIndex[PN] = -1;
	lastWeapon[PN] = "JUNKY";
	
	ACS_NamedExecuteAlways("SpawnerPulseScript",0,0,0,0);
	ACS_NamedExecuteAlways("LastWeaponHandler",0,0,0,0);
	ACS_NamedExecuteAlways("PlayerTicker",0,0,0,0);
	GiveStartingItems();
	
	UpdateAlive();
	
	// S->C: Create our HUD
	ACS_NamedExecuteAlways("RequestHUDCreation", 0);
}

// ------------------------/
// GIVE STARTING WEAPONS
// ------------------------/

void GiveStartingItems()
{
	ClearInventory();
	
	// Give our pistol first, on purpose
	// This SHOULD take priority over the others
	
	// (If we don't do this, the client may select it but the server might not)
	
	GiveInventory("CanShowHUD", 1);
	
	GiveInventory("TestPistol", 1);
	GiveInventory("Type97Grenade", 1);
	
	GiveInventory("Fist", 1);
	GiveInventory("Syringe", 1);
	
	SetWeapon("TestPistol");
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

script "PlayerJoin" ENTER { PlayerJoined(); }
script "PlayerRespawn" RESPAWN { SetupPhysicalPlayer(); }
script "PlayerDeath" DEATH { PlayerDied(); }
script "PlayerDisconnect" (int gone) DISCONNECT { PlayerDisconnect(gone); }

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Update all living players! See who's alive and who's not
// ------------------------/

void UpdateAlive()
{
	int aliveCount = 0;
	
	for (int l=0; l<MAXPLAYERS; l++)
	{
		int TID = PlayerTID(l);
		
		bool lastAlive = PlayerState[l].Alive;
		
		if (PlayerInGame(l) && PlayerIsSpectator(l) == 0 && (ClassifyActor(TID) & ACTOR_ALIVE))
		{
			PlayerState[l].Alive = true;
			aliveCount ++;
		}
		else
			PlayerState[l].Alive = false;
		
		// The state changed! Sync it to clients
		if (PlayerState[l].Alive != lastAlive)
		{
			Rep_UpdatePlayerState(l, -1);
		}
	}
	
	// Looks like everyone is DEAD!!!
	if (aliveCount <= 0 && GameState.State != STATE_LOBBY && GameState.State != STATE_SANDBOX)
	{
		SetActivator(-1);
		ACS_NamedExecute("SquadDied",0,0,0,0);
	}
}

// ------------------------/
// Respawn all dead players!
// ------------------------/

script "RevivePlayers" NET { RevivePlayers(); }
void RevivePlayers()
{
	for (int l=0; l<MAXPLAYERS; l++)
	{
		// This player is DEAD
		if (PlayerIsSpectator(l) == 2)
		{
			// This is important, because the clients receiving this
			// will close their lobby menu and whatnot
			Rep_UpdatePlayerState(l, PS_ALIVE);
			
			SetDeadSpectator(l, false);
			
			// Why 4? Who knows
			SetPlayerLivesLeft(l, 4);
		}
	}
}

// ------------------------/
// Are all players ready?
// ------------------------/

bool AllPlayersReady()
{
	int ReadyGoal = DesiredReadyPlayers();
	int ReadyCur = 0;
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerState[i].Ready)
			ReadyCur ++;
	}
	
	SetFont(FNT_PLAIN);
	PrintBold(d:ReadyCur, s:" / ", d:ReadyGoal, s:" READY");
	
	return (ReadyCur >= ReadyGoal);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// A player disconnected!
// ------------------------/

void PlayerDisconnect(int ID)
{
	// Don't perform this on dying spectators
	
	if (PlayerIsSpectator(ID) != 2)
	{
		Rep_UpdatePlayerState(ID, PS_RESET);
		UpdateAlive();
	}
}

// ------------------------/
// A player has spawned into the map for the first time!
// SERVER
// ------------------------/

void PlayerJoined()
{
	int PN = PlayerNumber();
	
	// Can the game be joined?
	if (GameIsJoinable())
		SetupPhysicalPlayer();
	else
	{
		// Start off as not being ready OR alive
		// This applies for lobby, mid-game things, etc.
		
		Rep_UpdatePlayerState(PN, PS_RESET);
		
		SetActivator(-1);
		
		SetDeadSpectator(PN, 1);
		
		UpdateAlive();
	}
}

// ------------------------/
// A PLAYER HAS DIED!
// ------------------------/

void PlayerDied()
{
	int PN = PlayerNumber();
	
	InvEnsure("CanShowHUD", 0);
	
	// Dying in trader is allowed, nowhere else
	if (GameState.State == STATE_TRADER)
		return;
		
	SetActivator(-1);
	
	// Update living players
	UpdateAlive();
		
	// Set us as a dead spectator
	SetDeadSpectator(PN, 1);
}

// ------------------------/
// ATTEMPT TO READY UP!
// CLIENT -> SERVER
// ------------------------/

script "ReadyUp" NET
{
	int PN = PlayerNumber();
	
	// We're ready, don't worry about it
	if (PlayerState[PN].Ready)
		terminate;
		
	PlayerState[PN].Ready = true;
	
	// If all players are ready, then start the game!
	// Don't bother updating ready state, things will
	// get sync'd when the players spawn and whatnot
	if (AllPlayersReady())
	{
		// Spawn all players if we're in the lobby state
		if (GameState.State == STATE_LOBBY)
			StartInvasionGame();
	}
	
	// Otherwise, just send our ready state to the players
	else
		Rep_UpdatePlayerState(PN, PS_READY);
}

// ------------------------/
// EVERYONE IS DEAD
// ------------------------/

script "SquadDied" (void)
{
	PrintBold(s:"\cGYOUR SQUAD WAS WIPED OUT");

	int cur = 0;
	int goal = 35*3;

	
	while (cur < goal && PlayerCount() > 0) { cur ++; delay(1); }
	
	// Players are still in the server, let's go to the next map
	if (cur >= goal)
		SquadEnd();
		
	// Hm, server is empty, let's just reset the map
	else
		ResetMap();
}

// ------------------------/
// Proceed to the next map!
// ------------------------/

void SquadEnd()
{
	Exit_Normal(0);
}

// ------------------------/
// Called every tick for each player
// ------------------------/

using ZTHealingCode;

script "PlayerTicker"
{
	while (ClassifyActor(0) & ACTOR_ALIVE)
	{
		Input_Cleanup();
		Input_Tick();
		Healing_Tick();
		delay(1);
	}
}