// -- Utility functions

// Ensure that we have X amount of an inventory item
void InvEnsure(str item, int amt)
{
	int cnt = CheckInventory(item);
	if (amt > cnt)
		GiveInventory(item, amt-cnt);
	else if (cnt > amt)
		TakeInventory(item, cnt - amt);
}

// Check if a button was pressed ONCE, not held
bool ButtonPressed(int input, int BT = -1, int OBT = -1)
{
	int ActNum = -1;
	
	if (IsLocal())
		ActNum = ConsolePlayerNumber();
	
	if (BT == -1)
		BT = GetPlayerInput(ActNum, INPUT_BUTTONS);
	if (OBT == -1)
		OBT = GetPlayerInput(ActNum, INPUT_OLDBUTTONS);
		
	if( (BT & input) && !(OBT & input) )
		return true;
		
	return false;
}

// Check if a button was released
bool ButtonReleased(int input, int BT = -1, int OBT = -1)
{
	int ActNum = -1;
	
	if (IsLocal())
		ActNum = ConsolePlayerNumber();
		
	if (BT == -1)
		BT = GetPlayerInput(ActNum, INPUT_BUTTONS);
	if (OBT == -1)
		OBT = GetPlayerInput(ActNum, INPUT_OLDBUTTONS);
		
	if( !(BT & input) && (OBT & input) )
		return true;
		
	return false;
}

// Create a "fake" hash
// This turns a string into a unique int based on characters
// Fake hashes should be the SAME on client and server
int FakeHash(str hasher)
{
	int hash = 0;
	for (int l=0; l<StrLen(hasher); l++)
		hash += GetChar(hasher, l) + (3*l);
		
	return hash;
}

// Generates CRC hash from text, roughly speaking
// Code from krystalgamer's SM2000 save editor

// This should HEAVILY REDUCE CONFLICTS

int CRC(str hasher)
{
	int sum = 0;
	
	for (int i=0; i<StrLen(hasher); i++)
	{
		int chr = GetChar(hasher, i);
		
		if (sum & 0x80000000)
			sum ++;
		else
			sum <<= 1;
			
		sum += chr;
	}
	
	sum |= 1;
	
	return sum;
}

// Half a fixed
int Half(int num)
{
	return FixedMul(num, 0.5);
}

// Smallest of 2 values
int min (int a, int b)
{
	if (a < b)
		return a;

	return b;
}

// Largest of 2 values
int max (int a, int b)
{
	if (a > b)
		return a;

	return b;
}

// Evaluate a string
str StrEval(int cond, str a, str b)
{
	if (cond)
		return b;

	return a;
}

// Sandbox mode
bool IsSandbox()
{
	return (ThingCountName("Sandboxer", 0) > 0);
}

// We're playing alone
bool IsOffline()
{
	return !GameState.Online;
}

// Local player
bool IsLocal()
{
	return (GameState.Singleplayer || GameState.Client);
}

// Unreal-like AUTHORITY check
bool IsAuthority()
{
	return (GameState.Singleplayer || !GameState.Client);
}

// ONLY on the server! For network games only
bool IsServer()
{
	return (!GameState.Client && GameState.Online);
}

// ONLY on the client! For network games only
bool IsClient()
{
	return (GameState.Client && GameState.Online);
}

// Ensure we're the correct client, for S->C scripts
bool WrongClient()
{
	return (PlayerNumber() != ConsolePlayerNumber());
}

// Quick and lazy proximity check
bool ActorInRange(int XV, int YV, int ZV, int tid2, int range)
{
	int x, y, z;
	int xd, yd, zd;
	
	x = GetActorX(tid2);
	y = GetActorY(tid2);
	z = GetActorZ(tid2);
	
	xd = abs(XV - x);
	if (xd <= range)
		return true;
		
	yd = abs(YV - y);
	if (yd <= range)
		return true;
		
	return false;
}

// Lazy distance between two TID's, AS AN INTEGER
int lazyDistance (int tida, int tidb)
{
	int xa, ya, xb, yb;
	
	xa = ShaveInt(GetActorX(tida));
	ya = ShaveInt(GetActorY(tida));
	
	xb = ShaveInt(GetActorX(tidb));
	yb = ShaveInt(GetActorY(tidb));
	
	int xDist = (xa + xb) / 2;
	int yDist = (xa + xb) / 2;
	
	return (xDist + yDist) / 2;
}

// Distance betwene a point and a TID
int distancePoint (int XV, int YV, int ZV, int tid2)
{
	int x, y, z, d;
	x = XV - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = YV - GetActorY(tid2) >> 16;
	z = ZV - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

// Slightly slower version of distancePoint, but has no sqrt looping apparently
int fdistancePoint (int XV, int YV, int ZV, int tid2)
{
	int len;
	int y = YV - getactory(tid2);
	int x = XV - getactorx(tid2);
	int z = ZV - getactorz(tid2);

	int ang = vectorangle(x,y);
	if(((ang+0.125)%0.5) > 0.25) len = fixeddiv(y, sin(ang));
	else len = fixeddiv(x, cos(ang));

	ang = vectorangle(len, z);
	if(((ang+0.125)%0.5) > 0.25) len = fixeddiv(z, sin(ang));
	else len = fixeddiv(len, cos(ang));

	return len >> 16;
}

// Distance between two TIDs
int distance (int tid1, int tid2)
{
	int x, y, z, d;
	x = GetActorX(tid1) - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = GetActorY(tid1) - GetActorY(tid2) >> 16;
	z = GetActorZ(tid1) - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

bool Between(int num, int a, int b)
{
	return (num >= a && num <= b);
}

int clamp(int a, int mn, int mx)
{
	if (a >= mx)
		return mx;
	else if (a <= mn)
		return mn;
	else
		return a;
}

int abs(int x) { return (x < 0) ? -x : x; }

// Cast to integer
int ShaveInt(int num) { return (num>>16); }
int Shave(int num) { return (num>>16)<<16; }

// Cast to float, with fractional removed
int ToFloat(int num) { return (num>>16)<<16; }

// We're waiting to respawn (Called locally)
bool WaitingToRespawn()
{
	int CPN = IsLocal() ? ConsolePlayerNumber() : PlayerNumber();
	
	// Lobby?
	if (GameState.State == STATE_LOBBY)
		return false;
	
	// Dead, ready, game cannot be joined
	if (PlayerIsSpectator(CPN) == 2 && PlayerState[CPN].Ready && !GameIsJoinable() )
		return true;
		
	return false;
}

// Check if a player is alive, in the game, etc.
bool PlayerValid(int PN)
{
	return (PlayerInGame(PN) || PlayerIsSpectator(PN));
}

// Obtain player TID based on number
int PlayerTID(int num)
{
	return BASE_PLAYER_TID + num;
}

// Lists all ALIVE or DEAD players that need to be ready for the game to start
// (Plain spectators don't count)

int DesiredReadyPlayers()
{
	int count = 0;
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerInGame(i) || PlayerIsSpectator(i) == 2)
			count ++;
	}

	return count;
}

// Pad a string to a certain length
str StrPad(str string, str padder, int len)
{
	while (StrLen(string) < len)
		string = StrParam(s:padder, s:string);
		
	return string;
}

// Return a padded time string from a number
str TimeString(int sec)
{
	// Split the trader text into minutes
	int minutes = ShaveInt(FixedDiv(sec*1.0,60.0));
	str minuteText = StrPad( StrParam(d:minutes), "0", 2 );
		
	// Split the trader text into seconds
	int seconds = sec - (60*minutes);
	str secondText = StrPad( StrParam(d:seconds), "0", 2 );

	return StrParam(s:minuteText,s:":",s:secondText);
}

// Wraps a string and inserts newlines as needed
// This is fairly expensive!
str StrWrap(str strBase, int wrapWidth, int glyphSize)
{
	str newString = "";
	int lineWidth = 0.0;
	int LN = StrLen(strBase);
	int WrappedLines = 1;
	bool StoredDash;
	
	for (int i=0; i<LN; i++)
	{
		str CHR = StrMid(strBase, i, 1);
		if (CHR == "\t")
				continue;
				
		// This is ABSOLUTELY a newline
		bool NL = (CHR == "\n");
		
		// It's a newline, or it's a space! We can split!
		if (NL || (LineWidth >= wrapWidth && CHR == " "))
		{
			LineWidth = 0.0;
			WrappedLines ++;
			StoredDash = false;
			newString = StrParam(s:newString, s:"\n");
		}
		
		// Hey, this isn't a newline! Must be a normal letter
		if (!NL)
		{
			// Our line starts with a -
			if (LineWidth == 0.0 && CHR == "-")
				StoredDash = true;
				
			newString = StrParam(s:newString, s:CHR);
			LineWidth += glyphSize;
		}
	}
	
	GameState.HUD.LastWrappedLines = WrappedLines;
	return newString;
}

// Freeze / Unfreeze a player
void FreezeMe(bool frozen, int num = 0)
{
	SetActorProperty(num,APROP_Invulnerable,frozen);
	SetPlayerProperty(num,frozen,4);
	SetPlayerProperty(num,frozen,1);
	
	// Invis
	// SetPlayerProperty(num,frozen,7);
}

// Get a normalized -1.0 to 1.0 value for whether or not you're facing something
int dotFacing(int tidA, int tidB, int angleMod = 0.0)
{
	int xA = GetActorX(tidA);
	int yA = GetActorY(tidA);
	
	int xB = GetActorX(tidB);
	int yB = GetActorY(tidB);
	
	int angA = GetActorAngle(tidA) + angleMod;
	
	// Vector A
	int x1 = xB - xA;
	int y1 = yB - yA;
	
	// Normalize
	int l = VectorLength(x1, y1);
	x1 = FixedDiv(x1, l);
	y1 = FixedDiv(y1, l);
	
	// Vector B
	int x2 = cos(angA);
	int y2 = sin(angA);
	
	return FixedMul(x1, x2) + FixedMul(y1, y2);
}

// Spawn a cinematic camera facing something, at a specific angle
// Returns the TID of the cam
// (sightChecker will fail if CheckSight is false (good for noclip))

int SpawnCinematicCamera(int x, int y, int z, int ang, int sightChecker = -1)
{
	int tryX = x + FixedMul(CINE_DISTANCE, cos(ang));
	int tryY = y + FixedMul(CINE_DISTANCE, sin(ang));
	int pointX = x - tryX;
	int pointY = y - tryY;
	
	// Get the angle the camera SHOULD be facing (as byte angle)
	int pointAng = VectorAngle(pointX, pointY) >> 8;
	
	int UID = UniqueTID();
	if (Spawn("CinematicTester", tryX, tryY, z + 40.0, UID, pointAng))
	{
		// Line of sight test, fail if it doesn't work
		if (sightChecker >= 0)
		{
			if (!CheckSight(UID, sightChecker, 0))
			{
				Thing_Destroy(UID, 1, 0);
				return 0;
			}
		}
		
		// Get its pos, then KILL IT
		x = GetActorX(UID);
		y = GetActorY(UID);
		z = GetActorZ(UID);

		Thing_Destroy(UID, 1, 0);
		
		Spawn("AimingCamera", x, y, z, UID, pointAng);
		
		return UID;
	}
	
	return 0;
}

// Attempt to FORCEFULLY spawn a cinematic camera
// If the initial spawn fails, this will try multiple angles

int SpawnCinematicCameraForced(int x, int y, int z, int ang, int sightChecker = -1)
{
	int initialCam = SpawnCinematicCamera(x, y, z, ang, sightChecker);
	
	// Success?
	if (initialCam)
		return initialCam;
		
	// Looks like it didn't work, let's start checking angles
	int chopper = FixedDiv(1.0, CINE_ANGLE_CHOPS * 1.0);
	
	for (int i=0; i<CINE_ANGLE_CHOPS; i++)
	{
		int testAngle = FixedMul(i*1.0, chopper);
		int testCamera = SpawnCinematicCamera(x, y, z, testAngle, sightChecker);
		
		if (testCamera)
			return testCamera;
	}
	
	// Oh dear, we still don't have a camera
	return 0;
}

// Freeze all players
void FreezeAllPlayers(bool total = false, bool stopInPlace = false)
{
	int ac = ActivatorTID();
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerValid(i))
		{
			SetActivator( PlayerTID(i) );
			SetPlayerProperty(0, 1, total ? PROP_TOTALLYFROZEN : PROP_FROZEN);
			
			if (stopInPlace)
				Thing_Stop(ActivatorTID());
		}
	}
	
	SetActivator(ac);
}

// Give all players invulnerability
void InvulAllPlayers()
{
	int ac = ActivatorTID();
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerValid(i))
		{
			SetActivator( PlayerTID(i) );
			SetPlayerProperty(0, 2, PROP_INVULNERABILITY);
		}
	}
	
	SetActivator(ac);
}

// Converts character to uppercase
int ToUpper(int c)
{
	if (c >= 97 && c <= 122)
		return c - 32;
		
	return c;
}

// Uppercases a string
str StrCapitalize(str s)
{
	if (StrLen(s) == 0)
		return "";

	str result = "";
	int len = StrLen(s);
	
	for (int i = 0; i < len; i++)
		result = StrParam(s:result, c:ToUpper(GetChar(s, i)));

	return result;
}

// Check to see if a command is unbound
// (It has no key bound to it!)

bool IsUnbound(str cmd)
{
	str keyString = StrParam(k:cmd);
	str qString = StrLeft(keyString, 3);
	
	return (StrCmp(qString, "???") == 0);
}